#ifndef __VARS_H__
#define __VARS_H__

typedef struct _ircd_var	IRCD_VAR;

#include "defines.h"
#include "structs.h"

int vars_logfilename_check(IRCD_CLIENT *pptr, IRCD_VAR *v, char *value);
int vars_motdfilename_check(IRCD_CLIENT *pptr, IRCD_VAR *v, char *value);
int vars_klinesfilename_check(IRCD_CLIENT *pptr, IRCD_VAR *v, char *value);
int vars_glinesfilename_check(IRCD_CLIENT *pptr, IRCD_VAR *v, char *value);
int vars_whowas_change(IRCD_CLIENT *pptr, IRCD_VAR *v);
int vars_logfile_change(IRCD_CLIENT *pptr, IRCD_VAR *v);
int vars_isupport_change(IRCD_CLIENT *pptr, IRCD_VAR *v);

struct _ircd_var
{
	char	*v_name;

#define IRCD_VAR_TYPE_INT		1
#define IRCD_VAR_TYPE_STRING	2
#define IRCD_VAR_TYPE_BOOL		3

	char	v_type;

#define IRCD_VAR_FLAGS_CONFONLY		0x001
	u_int	v_flags;

	int		v_val_min;	/* for ints, minimum value..for string, min length */
	int		v_val_max;	/* for ints, maximum value..for string, max length */

	void	*v_value;

	int		(*v_functionbefore)(IRCD_CLIENT *pptr, IRCD_VAR *vptr, char *value);
	int		(*v_functionafter)(IRCD_CLIENT *pptr, IRCD_VAR *vptr);
};

#define IRCD_VAR_CHANOPSONSPLIT				0
#define IRCD_VAR_MAXCLIENTS					1
#define IRCD_VAR_IDLE_CHECK					2
#define IRCD_VAR_IDLE_TIME					3
#define IRCD_VAR_GLINE_LOG					4
#define IRCD_VAR_GLINE_LOGFILE				5
#define IRCD_VAR_GOODOPER_LOG				6
#define IRCD_VAR_GOODOPER_LOGFILE			7
#define IRCD_VAR_BADOPER_LOG				8
#define IRCD_VAR_BADOPER_LOGFILE			9
#define IRCD_VAR_SHOW_HEADERS				10
#define IRCD_VAR_SHOW_NICKCHANGES			11
#define IRCD_VAR_NEWKLINES_FILE				12
#define IRCD_VAR_NEWGLINES_FILE				13
#define IRCD_VAR_MOTD_FILE					14
#define IRCD_VAR_REMOTE_KLINES				15
#define IRCD_VAR_CLONES_LOG					16
#define IRCD_VAR_CLONES_LOGFILE				17
#define IRCD_VAR_USERS_LOG					18
#define IRCD_VAR_USERS_LOGFILE				19
#define IRCD_VAR_USERNAMECHARS				20
#define IRCD_VAR_ISOPERSTRING				21
#define IRCD_VAR_CLIENTCONNECT				22
#define IRCD_VAR_IPADDR_ACCEPT_RATE_NUM		23
#define IRCD_VAR_IPADDR_ACCEPT_RATE_TIME	24
#define IRCD_VAR_IPADDR_CLIENT_RATE_NUM		25
#define IRCD_VAR_IPADDR_CLIENT_RATE_TIME	26
#define IRCD_VAR_WHOWAS_SIZE				27
#define IRCD_VAR_RATELIMIT_TIME				28
#define IRCD_VAR_RATELIMIT_MOTD_NUM			29
#define IRCD_VAR_RATELIMIT_INFO_NUM			30
#define IRCD_VAR_RATELIMIT_STATS_NUM		31
#define IRCD_VAR_REJECT_HOLD				32
#define IRCD_VAR_PRIORITY_IDLE				33
#define IRCD_VAR_PRIORITY_LAG				34
#define IRCD_VAR_PRIORITY_FDLISTS			35
#define IRCD_VAR_DEFAULT_MAXCHANNELS		36
#define IRCD_VAR_MAXUNKLINES				37
#define IRCD_VAR_OPERSCANJOINONSPLIT		38
#define IRCD_VAR_SHOW_CLIENTCOUNTER			39
#define IRCD_VAR_IGNORE_REDUNDANT_MODES		40
#define IRCD_VAR_MAX_MODE_PARAMS			41
#define IRCD_VAR_MAX_BANS					42
#define IRCD_VAR_MAX_KICK_REASON_LEN		43
#define IRCD_VAR_MAX_KILL_REASON_LEN		44
#define IRCD_VAR_MAX_QUIT_REASON_LEN		45
#define IRCD_VAR_MAX_TOPIC_LEN				46
#define IRCD_VAR_MAX_AWAY_LEN				47
#define IRCD_VAR_MAX_TARGETS				48
#define IRCD_VAR_IGNORE_GLINES				49
#define IRCD_VAR_CAN_GLINE					50
#define IRCD_VAR_GLINE_VOTES_NEEDED			51
#define IRCD_VAR_GLINE_TRIGGERED_TIMEOUT	52
#define IRCD_VAR_GLINE_UNTRIGGERED_TIMEOUT	53

#ifdef DEFINE_VARS_DEFINES

IRCD_VAR	Vars_Table[] =
{
/*      name                type       flags  min   max   default value */

	{ "CHANOPSONSPLIT",	IRCD_VAR_TYPE_INT, 0, 0,	999, (void *)0, NULL, NULL },
	{ "MAXCLIENTS",		IRCD_VAR_TYPE_INT,	0, 0,	0, (void *)1024, NULL, NULL },
	{ "IDLE_CHECK",		IRCD_VAR_TYPE_BOOL,	0, 0,	0,	(void *)0, NULL, NULL },
	{ "IDLE_TIME",		IRCD_VAR_TYPE_INT,	0, 300,0,	(void *)300, NULL, NULL },
	{ "GLINE_LOG",		IRCD_VAR_TYPE_BOOL,	0, 0,	0,	(void *)0, NULL, vars_logfile_change },
	{ "GLINE_LOGFILE",	IRCD_VAR_TYPE_STRING,	0, 0,	255, "logs/gline.log", vars_logfilename_check, vars_logfile_change },
	{ "GOODOPER_LOG",		IRCD_VAR_TYPE_BOOL,	0, 0,	0,	(void *)0, NULL, vars_logfile_change },
	{ "GOODOPER_LOGFILE",	IRCD_VAR_TYPE_STRING,	0, 0,	255, "logs/good-opers.log", vars_logfilename_check, vars_logfile_change },
	{ "BADOPER_LOG",		IRCD_VAR_TYPE_BOOL,	0, 0,	0,	(void *)0, NULL, vars_logfile_change },
	{ "BADOPER_LOGFILE",	IRCD_VAR_TYPE_STRING,	0, 0,	255, "logs/bad-opers.log", vars_logfilename_check, vars_logfile_change },
	{ "SHOW_HEADERS",	IRCD_VAR_TYPE_BOOL, 0, 0,	0,	(void *)0, NULL, NULL },
	{ "SHOW_NICKCHANGES",	IRCD_VAR_TYPE_BOOL,	0, 0,	0,	(void *)0, NULL, NULL },
	{ "NEWKLINES_FILE",	IRCD_VAR_TYPE_STRING,	0, 0,	255,(void *)0, vars_klinesfilename_check, NULL },
	{ "NEWGLINES_FILE",	IRCD_VAR_TYPE_STRING,	0, 0,	255,(void *)0, vars_glinesfilename_check, NULL },
	{ "MOTD_FILE",		IRCD_VAR_TYPE_STRING,	0, 0,	255,	"ircd.motd", vars_motdfilename_check, NULL },
	{ "REMOTE_KLINES",	IRCD_VAR_TYPE_BOOL,	0, 0,	0,	(void *)0, NULL, NULL },
	{ "CLONES_LOG",		IRCD_VAR_TYPE_BOOL,	0, 0,	0,	(void *)0, NULL, vars_logfile_change },
	{ "CLONES_LOGFILE",	IRCD_VAR_TYPE_STRING,	0, 0,	255, "logs/clones.log", vars_logfilename_check, vars_logfile_change },
	{ "USERS_LOG",		IRCD_VAR_TYPE_BOOL,	0, 0,	0,	(void *)0, NULL, vars_logfile_change },
	{ "USERS_LOGFILE",	IRCD_VAR_TYPE_STRING,	0, 0,	255, "logs/users.log", vars_logfilename_check, vars_logfile_change },
	{ "USERNAMECHARS",	IRCD_VAR_TYPE_STRING,	0, 0,	0,	(void *)"-_", NULL, NULL },
	{ "ISOPERSTRING",	IRCD_VAR_TYPE_STRING,	0, 1, 80, "is an IRC Operator", NULL, NULL },
	{ "CLIENTCONNECT",	IRCD_VAR_TYPE_BOOL,	0,	0, 0,	(void *)0, NULL, NULL },
	{ "IPADDR_ACCEPT_RATE_NUM",	IRCD_VAR_TYPE_INT,	0, 0,	0, (void *)20, NULL, NULL },
	{ "IPADDR_ACCEPT_RATE_TIME",	IRCD_VAR_TYPE_INT,	0, 0, 0, (void *)10, NULL, NULL },
	{ "IPADDR_CLIENT_RATE_NUM",	IRCD_VAR_TYPE_INT,	0, 0,	0, (void *)3, NULL, NULL },
	{ "IPADDR_CLIENT_RATE_TIME",	IRCD_VAR_TYPE_INT,	0, 0, 0, (void *)10, NULL, NULL },
	{ "WHOWAS_SIZE",	IRCD_VAR_TYPE_INT,	0, 1,	100000,	(void *)(16*1024), NULL, vars_whowas_change },
	{ "RATELIMIT_TIME",			IRCD_VAR_TYPE_INT,	0, 0, 0, (void *)6, NULL, NULL },
	{ "RATELIMIT_MOTD_NUM",		IRCD_VAR_TYPE_INT,	0, 0, 0, (void *)3, NULL, NULL },
	{ "RATELIMIT_INFO_NUM",		IRCD_VAR_TYPE_INT,	0, 0, 0, (void *)3, NULL, NULL },
	{ "RATELIMIT_STATS_NUM",	IRCD_VAR_TYPE_INT,	0, 0, 0, (void *)3, NULL, NULL },
	{ "REJECT_HOLD",			IRCD_VAR_TYPE_INT, 0, 0, 3600, (void *)0, NULL, NULL },
	{ "PRIORITY_IDLE",			IRCD_VAR_TYPE_INT,	0, 120, 0, (void *)(20*60), NULL, NULL },
	{ "PRIORITY_LAG",			IRCD_VAR_TYPE_INT,	0, 1, 100, (void *)3, NULL, NULL },
	{ "PRIORITY_FDLISTS",		IRCD_VAR_TYPE_INT,	0, 0, 99, (void *)4, NULL, NULL },
	{ "MAXCHANNELS",			IRCD_VAR_TYPE_INT,	0, 0, MAXCHANNELSPERUSER, (void *)DEFAULTMAXCHANNELSPERUSER, NULL, NULL },
	{ "MAXUNKLINES",			IRCD_VAR_TYPE_INT,	0, 0, 0, (void *)20, NULL, NULL },
	{ "OPERSCANJOINONSPLIT",	IRCD_VAR_TYPE_BOOL,	0, 0,	0,	(void *)0, NULL, NULL },
	{ "SHOW_CLIENTCOUNTER",		IRCD_VAR_TYPE_BOOL,	0, 0,	0,	(void *)1, NULL, NULL },
	{ "IGNORE_REDUNDANT_MODES",	IRCD_VAR_TYPE_BOOL,	0, 0,	0,	(void *)0, NULL, NULL },
	{ "MAX_MODE_PARAMS",		IRCD_VAR_TYPE_INT,	IRCD_VAR_FLAGS_CONFONLY, 2, MAXMODEPARAMS, (void *)DEFAULTMAXMODEPARAMS, NULL, vars_isupport_change },
	{ "MAX_BANS",				IRCD_VAR_TYPE_INT,	IRCD_VAR_FLAGS_CONFONLY, 10, MAXBANS, (void *)DEFAULTMAXBANS, NULL, vars_isupport_change },
	{ "MAX_KICK_REASON_LENGTH",		IRCD_VAR_TYPE_INT,	IRCD_VAR_FLAGS_CONFONLY, 80, MAXKICKLEN, (void *)DEFAULTMAXKICKLEN,	NULL, vars_isupport_change },
	{ "MAX_KILL_REASON_LENGTH",		IRCD_VAR_TYPE_INT,	IRCD_VAR_FLAGS_CONFONLY, 80, MAXKILLLEN, (void *)DEFAULTMAXKILLLEN,	NULL, vars_isupport_change },
	{ "MAX_QUIT_REASON_LENGTH",		IRCD_VAR_TYPE_INT,	IRCD_VAR_FLAGS_CONFONLY, 80, MAXQUITLEN, (void *)DEFAULTMAXQUITLEN,	NULL, vars_isupport_change },
	{ "MAX_TOPIC_LENGTH",			IRCD_VAR_TYPE_INT,	IRCD_VAR_FLAGS_CONFONLY, 80, MAXTOPICLEN, (void *)DEFAULTMAXTOPICLEN,	NULL, vars_isupport_change },
	{ "MAX_AWAY_LENGTH",			IRCD_VAR_TYPE_INT,	IRCD_VAR_FLAGS_CONFONLY, 80, MAXAWAYLEN, (void *)DEFAULTMAXAWAYLEN,	NULL, vars_isupport_change },
	{ "MAX_TARGETS",				IRCD_VAR_TYPE_INT,	IRCD_VAR_FLAGS_CONFONLY, 80, MAXTARGETS, (void *)DEFAULTMAXTARGETS,	NULL, vars_isupport_change },
	{ "IGNORE_GLINES",				IRCD_VAR_TYPE_BOOL,	0, 0, 0, (void *)0,	NULL, NULL },
	{ "CAN_GLINE",					IRCD_VAR_TYPE_BOOL,	0, 0, 0, (void *)0,	NULL, NULL },
	{ "GLINE_VOTES_NEEDED",			IRCD_VAR_TYPE_INT,	IRCD_VAR_FLAGS_CONFONLY, 2, 999, (void *)DEFAULT_GLINE_VOTES_NEEDED,	NULL, NULL },
	{ "GLINE_TRIGGERED_TIMEOUT",	IRCD_VAR_TYPE_INT,	0, 60, -1, (void *)DEFAULT_GLINE_TRIGGERED_TIMEOUT,	NULL, NULL },
	{ "GLINE_UNTRIGGERED_TIMEOUT",	IRCD_VAR_TYPE_INT,	0, 60, -1, (void *)DEFAULT_GLINE_UNTRIGGERED_TIMEOUT,	NULL, NULL },
	{ NULL, 0, 0, 0, 0, NULL }
};

#else

extern IRCD_VAR Vars_Table[];

#endif

#define vars_value_get_bynum(__x)	Vars_Table[(__x)].v_value

void vars_init(void);
IRCD_VAR *vars_find_byname(char *name);
void *vars_value_get_byname(char *name);
int vars_value_set_byname(IRCD_CLIENT *pptr, char *name, void *value);
int vars_value_set_bynum(IRCD_CLIENT *pptr, int num, void *value);
IRCD_VAR *vars_value_getall(void);
IRCD_VAR *vars_value_getnext(IRCD_VAR *v);
int vars_init_reload(void *varsptr);
int vars_deinit_reload(void **varsptr);

#endif
